"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.withGraphql = exports.getIsolatedQuery = exports.setOptions = exports.getOptions = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutPropertiesLoose"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _react = _interopRequireDefault(require("react"));

var _graphqlTag = _interopRequireDefault(require("graphql-tag"));

var _apolloBoost = _interopRequireDefault(require("apollo-boost"));

var traverse = _interopRequireWildcard(require("traverse"));

var _lodash = _interopRequireDefault(require("lodash.clonedeep"));

const options = new Map();

const getOptions = name => {
  if (!options.has(name)) {
    if (typeof window !== 'undefined') {
      setOptions(name, window.___graphqlUniversal[name]);
    }
  }

  return options.get(name);
};

exports.getOptions = getOptions;

const setOptions = (name, opts) => {
  if (!opts) {
    throw new Error('GraphQL Universal: No options "' + name + '".');
  }

  if (!opts.client && !opts.url) {
    throw new Error('GraphQL Universal: Could not get "url" for "' + name + '".');
  }

  if (!opts.typeName) {
    throw new Error('GraphQL Universal: Could not get "typeName" for "' + name + '".');
  }

  if (!opts.client) {
    opts.client = new _apolloBoost.default({
      uri: opts.url
    });
  }

  options.set(name, opts);
};

exports.setOptions = setOptions;

const getIsolatedQuery = (query, fieldName, typeName) => {
  const updatedQuery = (0, _lodash.default)(query);
  const updatedRoot = updatedQuery.definitions[0].selectionSet.selections.find(selection => selection.name && selection.name.kind === 'Name' && selection.name.value === fieldName);

  if (!updatedRoot) {
    console.warn('Failed to update query root');
    return;
  }

  updatedQuery.definitions[0].selectionSet.selections = updatedRoot.selectionSet.selections;
  traverse(updatedQuery).forEach(function (x) {
    if (this.isLeaf && this.parent && this.parent.key === 'name') {
      if (this.parent.parent && this.parent.parent.node.kind === 'NamedType') {
        if (typeof x === 'string' && x.indexOf(`${typeName}_`) === 0) {
          this.update(x.substr(typeName.length + 1));
        }
      }
    }
  });
  return updatedQuery;
};

exports.getIsolatedQuery = getIsolatedQuery;

const withGraphql = WrappedComponent => {
  var _temp;

  return _temp = class extends _react.default.Component {
    constructor(...args) {
      super(...args);
      (0, _defineProperty2.default)(this, "state", {
        data: this.props.data
      });
      (0, _defineProperty2.default)(this, "graphql", (fieldName, _ref) => {
        let query = _ref.query,
            _ref$composeData = _ref.composeData,
            composeData = _ref$composeData === void 0 ? true : _ref$composeData,
            queryProps = (0, _objectWithoutPropertiesLoose2.default)(_ref, ["query", "composeData"]);
        // Get options for graphql source plugin
        const options = getOptions(fieldName);

        if (!options || !options.client) {
          if (typeof window === 'undefined') {
            console.warn('GraphQL Universal: Options cannot be passed to plugin on server');
          } else {
            console.warn('GraphQL Universal: No options found for plugin "' + fieldName + '"');
          }

          return;
        }

        const client = options.client,
              typeName = options.typeName;

        if (query && query.source) {
          const updatedQuery = getIsolatedQuery((0, _graphqlTag.default)(query.source), fieldName, typeName);
          const rootValue = this.state.data && this.state.data[fieldName] || {};
          const res = client.query(Object.assign({
            query: updatedQuery,
            fetchPolicy: 'network-only'
          }, queryProps));

          if (composeData) {
            res.then(res => {
              this.setState({
                data: Object.assign({}, this.state.data, {
                  [fieldName]: Object.assign({}, rootValue, res.data)
                })
              });
            });
          }

          return res;
        } else if (query && query.kind && query.kind === 'Document') {
          return client.query(Object.assign({
            query,
            fetchPolicy: 'network-only'
          }, queryProps));
        }
      });
    }

    render() {
      return _react.default.createElement(WrappedComponent, (0, _extends2.default)({}, this.props, {
        graphql: this.graphql,
        data: this.state.data
      }));
    }

  }, _temp;
};

exports.withGraphql = withGraphql;